//
//  TwilioVoice.h
//  TwilioVoice
//
//  Copyright © 2016-2017 Twilio, Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "TVOAcceptOptions.h"
#import "TVOAudioDevice.h"
#import "TVOAudioFormat.h"
#import "TVOBaseTrackStats.h"
#import "TVOCall.h"
#import "TVOCallDelegate.h"
#import "TVOCallInvite.h"
#import "TVOCallOptions.h"
#import "TVOConnectOptions.h"
#import "TVODefaultAudioDevice.h"
#import "TVOError.h"
#import "TVOIceCandidatePairStats.h"
#import "TVOIceCandidateStats.h"
#import "TVOIceOptions.h"
#import "TVOLocalAudioTrackStats.h"
#import "TVOLocalTrackStats.h"
#import "TVONotificationDelegate.h"
#import "TVORemoteAudioTrackStats.h"
#import "TVORemoteTrackStats.h"
#import "TVOStatsReport.h"


/**
 * An enumeration indicating log levels that can be used with the SDK.
 */
typedef NS_ENUM(NSUInteger, TVOLogLevel) {
    TVOLogLevelOff = 0,     ///< Disables all SDK logging.
    TVOLogLevelFatal,       ///< Show very severe errors only.
    TVOLogLevelError,       ///< Show errors only.
    TVOLogLevelWarning,     ///< Show warnings as well as all **Error** log messages.
    TVOLogLevelInfo,        ///< Show informational messages as well as all **Warn** log messages.
    TVOLogLevelDebug,       ///< Show debugging messages as well as all **Info** log messages.
    TVOLogLevelTrace,       ///< Show low-level debugging messages as well as all **Debug** log messages.
    TVOLogLevelAll          ///< Show all logging.
};

/**
 * `TwilioVoice` logging modules.
 */
typedef NS_ENUM(NSUInteger, TVOLogModule) {
    TVOLogModuleCore = 0,   ///< Voice Core SDK.
    TVOLogModulePlatform,   ///< Voice iOS SDK.
    TVOLogModuleSignaling,  ///< Signaling module.
    TVOLogModuleWebRTC      ///< WebRTC module.
};

/**
 * `TwilioVoice` is the entry point to the Twilio Voice SDK. You can register for VoIP push notifications, make outgoing
 * Calls, receive CallInvites and audio device assignment using this class.
 */
@interface TwilioVoice : NSObject

/**
 * @brief The current logging level used by the SDK.
 *
 * @discussion The default logging level is `TVOLogLevelError`. `TwilioVoice` and its components use NSLog internally.
 *
 * @see TVOLogLevel
 */
@property (nonatomic, assign, class) TVOLogLevel logLevel;

/**
 *  @brief The `TVOAudioDevice` used to record and playback audio in a Call.
 *
 *  @discussion If you wish to provide your own `TVOAudioDevice` then you must set it before performing any other
 *  actions with the SDK like connecting to a Call. It is also possible to change the device when you are no longer
 *  connected to any Calls and have destroyed all other SDK objects.
 *
 *  @see [TVOAudioDevice](TVOAudioDevice.h)
 */
@property (class, nonatomic, strong, nonnull) id<TVOAudioDevice> audioDevice;

/**
 * @brief Returns the version of the SDK.
 *
 * @return The version of the SDK.
 */
+ (nonnull NSString *)sdkVersion;

/**
 * @brief Sets the logging level for an individual module.
 *
 * @param logLevel The `<TVOLogLevel>` level to be used by the module.
 * @param module The `<TVOLogModule>` for which the logging level is to be set.
 *
 * @see TVOLogLevel
 * @see TVOLogModule
 */
+ (void)setLogLevel:(TVOLogLevel)logLevel module:(TVOLogModule)module;

/**
 *  @brief Retrieve the log level for a specific module in the TwilioVoice SDK.
 *
 *  @param module The `TVOLogModule` for which the log level needs to be set.
 *
 *  @return The current log level for the specified module.
 */
+ (TVOLogLevel)logLevelForModule:(TVOLogModule)module;

/**
 * @name Managing VoIP Push Notifications
 */

/**
 * @brief Registers for Twilio VoIP push notifications.
 *
 * @discussion Registering for push notifications is required to receive incoming Calls through Twilio's infrastructure.
 * Once you've successfully registered it's not necessary to do so again unless your device token changes.
 *
 * @param accessToken Twilio Access Token.
 * @param deviceToken The push registry token for Apple VoIP Service.
 * @param completion Callback block to receive the result of the registration.
 */
+ (void)registerWithAccessToken:(nonnull NSString *)accessToken
                    deviceToken:(nonnull NSString *)deviceToken
                     completion:(nullable void(^)(NSError * __nullable error))completion;

/**
 * @brief Unregisters from Twilio VoIP push notifications.
 *
 * @discussion Call this method when you no longer want to receive push notifications for incoming Calls.
 * If your device token changes you should call this method to unregister your previous invalid token.
 *
 * @param accessToken Twilio Access Token.
 * @param deviceToken The push registry token for Apple VoIP Service.
 * @param completion Callback block to receive the result of the unregistration.
 */
+ (void)unregisterWithAccessToken:(nonnull NSString *)accessToken
                      deviceToken:(nonnull NSString *)deviceToken
                       completion:(nullable void(^)(NSError * __nullable error))completion;

/**
 * @brief Processes an incoming VoIP push notification payload.
 *
 * @discussion This method will asynchronously process your notification payload and call the provided delegate
 * on the main dispatch queue.
 *
 * @param payload Push notification payload.
 * @param delegate A `<TVONotificationDelegate>` to receive incoming push notification callbacks.
 *
 * @see TVONotificationDelegate
 */
+ (void)handleNotification:(nonnull NSDictionary *)payload
                  delegate:(nonnull id<TVONotificationDelegate>)delegate;


/**
 * @brief Connect an outgoing Call.
 *
 * @discussion This method is guaranteed to return a `<TVOCall>` object. It's possible for the returned Call to either
 * succeed or fail to connect.
 *
 * @param accessToken The access token.
 * @param delegate A `<TVOCallDelegate>` to receive Call state updates.
 *
 * @return A `<TVOCall>` object.
 *
 * @see TVOCall
 * @see TVOCallDelegate
 */
+ (nonnull TVOCall *)connectWithAccessToken:(nonnull NSString *)accessToken
                                   delegate:(nonnull id<TVOCallDelegate>)delegate;

/**
 * @brief Connect an outgoing Call.
 *
 * @discussion This method is guaranteed to return a `<TVOCall>` object. It's possible for the returned Call to either
 * succeed or fail to connect. Use the `<TVOConnectOptions>` builder to specify Call parameters and UUID
 *
 * @param options The connect options.
 * @param delegate A `<TVOCallDelegate>` to receive Call state updates.
 *
 * @return A `<TVOCall>` object.
 *
 * @see TVOCall
 * @see TVOCallDelegate
 * @see TVOConnectOptions
 */
+ (nonnull TVOCall *)connectWithOptions:(nonnull TVOConnectOptions *)options
                               delegate:(nonnull id<TVOCallDelegate>)delegate;

- (null_unspecified instancetype)init __attribute__((unavailable("TwilioVoice cannot be instantiated directly.")));

@end


/**
 * CallKit Audio Session Handling
 */
@interface TwilioVoice (CallKitIntegration)

@end
